/*
 * file Baseline.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList.ResponseIterator;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for a baseline resource.
 * 
 * A baseline resource is a version resource that captures the state of each
 * version-controlled member of a configuration. New baselines of a
 * configuration are created by checking out and then checking in the
 * configuration.
 * 
 * @since 1.0
 */
public interface Baseline extends Version {

    /**
     * A description of the difference between two sets of versions.
     * In the {@link Baseline#doCompareReport} method, the old set of versions is
     * specified by the object for which the method was invoked, and the new set of versions
     * is specified by the argument of the method.
     */
    public interface CompareReport { }

    /**
     * A version from a given version history that is in the new set of versions
     * but no version from the version history of that version is in the old set.
     */
    public interface AddedVersion extends CompareReport {

        /**
         * Return the added version.
         * 
         * @return the version of a resource added in the new set of versions.
         */
        public Version getVersion();
    }

    /**
     * A version from a given version history that is in the old set of versions
     * but no version from that version history is in the new set of versions.
     */
    public interface DeletedVersion extends CompareReport {

        /**
         * Return the deleted version.
         * 
         * @return the version of a resource removed from the new set of versions.
         */
        public Version getVersion();
    }

    /**
     * A version from a given version history that is in the old set of versions
     * where a different version from that version history is in the new set of versions.
     * @see Baseline#doCompareReport doCompareReport
     */
    public interface ChangedVersion extends CompareReport {

        /**
         * Return the version from the old baseline.
         * 
         * @return the version of the resource in the old baseline.
         */
        public Version getOldVersion();

        /**
         * Return the version from the new baseline.
         * 
         * @return the version of the resource in the new baseline.
         */
        public Version getNewVersion();
    }

    /**
     * An activity whose versions were added by the new set of versions.
     */
    public interface AddedActivity extends CompareReport {

        /**
         * Return the added activity.
         * 
         * @return the activity all of whose versions were added by the new set of versions.
         */
        public Activity getActivity();
    }

    /**
     * An activity some of whose versions were added by the new set of versions.
     */
    public interface PartiallyAddedActivity extends CompareReport {

        /**
         * Return the partially added activity.
         * 
         * @return the activity some of whose versions were added by the new set of versions.
         */
        public Activity getActivity();
    }

    /**
     * An activity whose versions were deleted by the new baseline.
     */
    public interface DeletedActivity extends CompareReport {

        /**
         * Return the deleted activity.
         * 
         * @return the activity whose versions were deleted by the new set of versions.
         */
        public Activity getActivity();
    }

    /**
     * An activity some of whose versions were deleted by the new set of versions.
     */
    public interface PartiallyDeletedActivity extends CompareReport {

        /**
         * Return the partially deleted activity.
         * 
         * @return the activity some of whose versions were deleted by the new set of versions.
         */
        public Activity getActivity();
    }

    /**
     * An activity whose versions were both added and removed by the new set of versions.
     */
    public interface ChangedActivity extends CompareReport {

        /**
         * Return the changed activity.
         * 
         * @return the activity some of whose versions were added and some of whose versions
         * were deleted by the new set of versions.
         */
        public Activity getActivity();
    }

    /** Boolean flags for the doCompareReport method */
    public static class CompareFlag
    {
        private CompareFlag(String flagName) { _flagName = flagName; }

        /**
         * Returns a string representation of this flag suitable for diagnostics.
         */
        @Override
        public String toString() { return _flagName; }

        /**
         * The string form of this flag.
         */
        private final String _flagName;

        /** 
         * Indicates that the comparison should include information about activities whose 
         * {@link Activity#ACTIVITY_VERSION_LIST} contain versions that are an ancestor of
         * an old version or an ancestor of a new version, but not both.
         * <p>
         * In particular, let NewVersions be the set of versions that are a new version or an ancestor of
         * a new version, but not an old version or an ancestor of an old version; and let OldVersions be the
         * set of versions that are an old version or an ancestor of an old version, but not a new version
         * or an ancestor of a new version.
         * Then an activity is considered:
         * <li>"added" if every version in its {@link Activity#ACTIVITY_VERSION_LIST} 
         * is in the NewVersions set.
         * <li>"deleted" if every version in its {@link Activity#ACTIVITY_VERSION_LIST} 
         * is in the OldVersions set.
         * <li>"partially added" if some versions in its {@link Activity#ACTIVITY_VERSION_LIST} 
         * are in the NewVersions set and the rest are in neither set.
         * <li>"partially deleted" if some versions in its {@link Activity#ACTIVITY_VERSION_LIST} 
         * are in the OldVersions set and the rest are in neither set.
         * <li>"changed" if some versions in its {@link Activity#ACTIVITY_VERSION_LIST} are in
         * the NewVersions set and some are in the OldVersions set.
         */
        public static final CompareFlag ACTIVITIES = new CompareFlag("activities"); //$NON-NLS-1$

        /**
         * Report only on versions in the new set of versions.
         * Changed versions are reported as added versions,
         * and deleted versions are not reported at all.
         */
        public static final CompareFlag NEW_ONLY = new CompareFlag("new-only"); //$NON-NLS-1$

    }


    /**
     * Compare two baselines.
     * 
     * @param baseline the baseline being compared to this Baseline.
     * @param flags boolean flags for the compare report.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the differences between the
     * versions selected by this Baseline and the baseline argument.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     * The resource identified by the <code>baseline</code> argument must be a baseline.
     */
    public ResponseIterator<CompareReport> doCompareReport(
            Baseline baseline,
            CompareFlag[] flags, 
            Feedback feedback)
    throws WvcmException;

    /**
     * Compare this Baseline with a stream.
     * 
     * @param stream the stream being compared to this Baseline.
     * @param flags boolean flags for the compare report.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the differences between the
     * versions selected by this Stream and the stream argument.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This resource does not support this report.
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     * The resource identified by the <code>stream</code> argument must be a stream.
     */
    public ResponseIterator<CompareReport> doCompareReport(
            Stream stream,
            CompareFlag[] flags, 
            Feedback feedback)
    throws WvcmException;

    /**
     * A list of all the versions that make up this baseline. The list is
     * created from all the {@link ControllableResource#CHECKED_IN} versions of
     * all the version-controlled members of the
     * {@link Configuration#ROOT_FOLDER} of the configuration at the time the
     * baseline identified by this Baseline was created.
     */
    public static final PropertyName<ResourceList<Version>> VERSION_LIST =
        new PropertyName<ResourceList<Version>>("version-list"); //$NON-NLS-1$

    /**
     * Get the {@link #VERSION_LIST} property.
     * 
     * @return the {@link #VERSION_LIST} property.
     * @throws WvcmException if this Baseline was not created with 
     * {@link #VERSION_LIST} as a wanted property.
     */
    public ResourceList<Version> getVersionList() throws WvcmException;

}
